/*
 * Copyright (C) 2013 Canonical, Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License version 3, as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
 * SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MIR_SURFACE_MANAGER_H
#define MIR_SURFACE_MANAGER_H

// std
#include <memory>

// Qt
#include <QObject>

// Mir
#include <mir_toolkit/common.h>

// local
#include "mirsurface.h"

namespace mir { namespace scene { class Surface; }}
namespace mir { namespace scene { class Session; }}
class ShellServerConfiguration;

namespace unitymir
{

class MirSurfaceManager : public QObject
{
    Q_OBJECT

public:
    static MirSurfaceManager* singleton();

    MirSurfaceManager(QObject *parent = 0);
    ~MirSurfaceManager();

    MirSurface *shellSurface() const;

    MirSurface *surfaceFor(std::shared_ptr<mir::scene::Surface> const& surface);

Q_SIGNALS:
    void surfaceCreated(MirSurface* surface);
    void surfaceDestroyed(MirSurface* surface);
    void shellSurfaceChanged(MirSurface* surface);
//    void surfaceResized(MirSurface*);
//    void fullscreenSurfaceChanged();

public Q_SLOTS:
    void sessionCreatedSurface(mir::scene::Session const* session, std::shared_ptr<mir::scene::Surface> const&);
    void sessionDestroyingSurface(mir::scene::Session const*, std::shared_ptr<mir::scene::Surface> const&);
    void shellSurfaceCreated(std::shared_ptr<mir::scene::Surface> const&);

    void surfaceAttributeChanged(mir::scene::Surface const*, MirSurfaceAttrib, int);

private:
    QHash<const mir::scene::Surface *, MirSurface *> m_surfaces;
    MirSurface* m_shellSurface;
    ShellServerConfiguration* m_mirServer;
    static MirSurfaceManager *the_surface_manager;
};

} // namespace unitymir

#endif // MIR_SURFACE_MANAGER_H
