/*
 * Copyright (C) 2013 Canonical, Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License version 3, as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
 * SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// local
#include "initialsurfaceplacementstrategy.h"
#include "logging.h"

// mir
#include "mir/scene/surface_creation_parameters.h"
#include "mir/geometry/rectangle.h"

namespace ms = mir::scene;
namespace msh = mir::shell;

InitialSurfacePlacementStrategy::InitialSurfacePlacementStrategy(std::shared_ptr<msh::DisplayLayout> const& displayLayout, QObject *parent)
  : QObject(parent),
    m_displayLayout(displayLayout)
{
}

InitialSurfacePlacementStrategy::~InitialSurfacePlacementStrategy()
{
}

ms::SurfaceCreationParameters
InitialSurfacePlacementStrategy::place(ms::Session const& session, ms::SurfaceCreationParameters const& requestParameters)
{
    using namespace mir::geometry;
    auto placedParameters = requestParameters;
    uint32_t x = 0;
    uint32_t y = 0;

    Q_EMIT requestPlacementForSession(&session, x, y);

    placedParameters.top_left = Point{ X{x}, Y{y} };

    return placedParameters;
}
