/*
 * Copyright (C) 2013 Canonical, Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License version 3, as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
 * SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "sessionlistener.h"
#include "logging.h"

namespace ms = mir::scene;

Q_DECLARE_METATYPE(std::shared_ptr<ms::Session>)

SessionListener::SessionListener(QObject *parent) :
    QObject(parent)
{
    DLOG("SessionListener::SessionListener (this=%p)", this);
    // need to register type to send over threads with signal/slot
    qRegisterMetaType<std::shared_ptr<ms::Session>>("std::shared_ptr<mir::shell::Session>");
}

SessionListener::~SessionListener()
{
    DLOG("SessionListener::~SessionListener (this=%p)", this);
}

// Mir's default SurfaceManager calls these methods with instance of ms::Session
// it is not a plain ms::Session
void SessionListener::starting(std::shared_ptr<ms::Session> const& session)
{
    DLOG("SessionListener::starting (this=%p, session=%p)", this, (void*)session.get());
    Q_EMIT sessionStarting(std::static_pointer_cast<ms::Session>(session));
}

void SessionListener::stopping(std::shared_ptr<ms::Session> const& session)
{
    DLOG("SessionListener::stopping (this=%p, session=%p)", this, (void*)session.get());
    Q_EMIT sessionStopping(std::static_pointer_cast<ms::Session>(session));
}

void SessionListener::focused(std::shared_ptr<ms::Session> const& session)
{
    DLOG("SessionListener::focused (this=%p, session=%p)", this, (void*)session.get());
    Q_EMIT sessionFocused(std::static_pointer_cast<ms::Session>(session));
}

void SessionListener::unfocused()
{
    DLOG("SessionListener::unfocused (this=%p)", this);
    Q_EMIT sessionUnfocused();
}

void SessionListener::surface_created(ms::Session& session, std::shared_ptr<ms::Surface> const& surface)
{
    DLOG("SessionListener::surface_created (this=%p, session=%p, surface=%p)", this, &session, (void*)surface.get());
    Q_EMIT sessionCreatedSurface(&session, surface);
}

void SessionListener::destroying_surface(ms::Session& session, std::shared_ptr<mir::scene::Surface> const& surface)
{
    DLOG("SessionListener::destroying_surface (this=%p, session=%p, surface=%p)", this, &session, (void*)surface.get());
    Q_EMIT sessionDestroyingSurface(&session, surface);
}
