/*
 * Copyright (C) 2013 Canonical, Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License version 3, as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
 * SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// local
#include "surfacecoordinator.h"
#include "logging.h"

// mir
#include <mir/scene/surface_creation_parameters.h>

namespace ms = mir::scene;
namespace msc = mir::scene;

SurfaceCoordinator::SurfaceCoordinator(std::shared_ptr<mir::scene::SurfaceCoordinator> const& surfaceCoordinator,
                             QObject *parent)
    : QObject(parent)
    , m_surfaceCoordinator(surfaceCoordinator)
{
    qRegisterMetaType<std::shared_ptr<ms::Surface>>("std::shared_ptr<mir::scene::Surface>");
}

SurfaceCoordinator::~SurfaceCoordinator()
{
}

std::shared_ptr<ms::Surface>
SurfaceCoordinator::add_surface(
        ms::SurfaceCreationParameters const& params,
        ms::Session *session,
        std::shared_ptr<mir::scene::SurfaceObserver> const& observer)
{
    DLOG("SurfaceCoordinator::create_surface");
    static const msc::DepthId shellSurfaceDepth{1};
    static const msc::DepthId oskSurfaceDepth{2};
    static bool shellSurfaceFound = false; //FIXME: Hack to work around fact unable to detect shell surface

    std::shared_ptr<ms::Surface> mirSurface;

    if (params.name == "Qml Phone Shell" && !shellSurfaceFound) { //TODO(greyback) Need to identify shell surface more securely
        DLOG("Shell depth");
        auto newParams = params;
        newParams.depth = shellSurfaceDepth;
        newParams.input_mode = mir::input::InputReceptionMode::receives_all_input;
        m_shellSurface = m_surfaceCoordinator->add_surface(newParams, session, observer);
        shellSurfaceFound = true;
        Q_EMIT shellSurfaceCreated(m_shellSurface);
        return m_shellSurface;
    } else if (params.name == "MaliitOnScreenKeyboard") { // FIXME: string hardcoded into maliit
        DLOG("OSK depth");
        auto newParams = params;
        newParams.depth = oskSurfaceDepth;
        return m_surfaceCoordinator->add_surface(newParams, session, observer);
    } else {
        DLOG("Default depth");
        return  m_surfaceCoordinator->add_surface(params, session, observer);
    }
}

void SurfaceCoordinator::remove_surface(std::weak_ptr<mir::scene::Surface> const& surface)
{
    DLOG("SurfaceCoordinator::remove_surface");
    m_surfaceCoordinator->remove_surface(surface);
}

void SurfaceCoordinator::raise(std::weak_ptr<mir::scene::Surface> const& surface)
{
    DLOG("SurfaceCoordinator::raise");
    m_surfaceCoordinator->raise(surface);
}

std::shared_ptr<mir::scene::Surface> SurfaceCoordinator::shellSurface() const
{
    return m_shellSurface;
}
