/*
 * Copyright (C) 2013 Canonical, Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License version 3, as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
 * SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SURFACECOORDINATOR_H
#define SURFACECOORDINATOR_H

#include <QObject>

#include "mir/scene/surface_coordinator.h"

Q_DECLARE_METATYPE(std::shared_ptr<mir::scene::Surface>)

class SurfaceCoordinator : public QObject, public mir::scene::SurfaceCoordinator
{
    Q_OBJECT

public:
    SurfaceCoordinator(std::shared_ptr<mir::scene::SurfaceCoordinator> const& surfaceCoordinator,
                  QObject *parent = 0);
    ~SurfaceCoordinator();

    //Called by Mir server
    std::shared_ptr<mir::scene::Surface> add_surface(
            mir::scene::SurfaceCreationParameters const& params,
            mir::scene::Session* session,
            std::shared_ptr<mir::scene::SurfaceObserver> const& observer) override;

    void remove_surface(std::weak_ptr<mir::scene::Surface> const& surface) override;
    void raise(std::weak_ptr<mir::scene::Surface> const& surface) override;

    // Called by shell
    std::shared_ptr<mir::scene::Surface> shellSurface() const;

Q_SIGNALS:
    void shellSurfaceCreated(std::shared_ptr<mir::scene::Surface> const& surface);

private:
    const std::shared_ptr<mir::scene::SurfaceCoordinator> m_surfaceCoordinator;
    std::shared_ptr<mir::scene::Surface> m_shellSurface;
};

#endif // SURFACECOORDINATOR_H
